program PcntApp;

{---------------------------------------------------------------------
Example application for the custom control PercentBar (PBar.DLL) of
the SMWCC 1.0 Custom Control Pack

Copyright (C) by Sebastian Modersohn
---------------------------------------------------------------------}

{$R PcntApp}

uses WinTypes, WinProcs, OWindows, ODialogs, BWCC,
     {the import unit for the pbar}
     PCnt, PcntCo;

{for loading the DLL}
const DLLName ='PBAR.DLL';
      em_DLLNotFound =1;

{some Id's}
      id_ABar        =201;
      id_Inc         =301;
      id_Dec         =302;

type
  PPBarWindow = ^TPBarWindow;
  TPBarWindow = object(TDlgWindow)
    ABar: PBar;
    constructor Init(AParent: PWindowsObject; AName: PChar);

    procedure Increase(var Msg: TMessage);
      virtual id_First + id_Inc;
    procedure Decrease(var Msg: TMessage);
      virtual id_First + id_Dec;

    procedure Ok(var Msg: TMessage);
      virtual id_First + id_OK;
    procedure Help(var Msg: TMessage);
      virtual id_First + 998;
  end;

{App that loads and frees the DLL and initializes the main window}
  PPBarApp = ^TPBarApp;
  TPBarApp = object(TApplication)
    Lib: THandle;
    constructor Init(AName: PChar);
    destructor Done; virtual;
    procedure InitMainWindow; virtual;
    procedure Error(ErrorCode: Integer); virtual;
  end;

{ TPBarWindow }

constructor TPBarWindow.Init(AParent: PWindowsObject; AName: PChar);
begin
  inherited Init(AParent, AName);
  {Inits the pbar, it is disposed by the parent automatically}
  ABar:=New(PBar, InitResource(@Self, id_ABar));
end;

procedure TPBarWindow.Increase(var Msg: TMessage);
begin
  {one way to increase the percent of the bar}
  ABar^.AddPercent(1);
end;

procedure TPBarWindow.Decrease(var Msg: TMessage);
begin
  {one way to decrease the percent of the bar}
  ABar^.AddPercent(-1);
end;

procedure TPBarWindow.Ok(var Msg: TMessage);
begin
  CloseWindow;
end;

procedure TPBarWindow.Help(var Msg: TMessage);
begin
  WinHelp(HWindow, HelpFile, Help_Context, 100);
end;

{ TPBarApp }

constructor TPBarApp.Init(AName: PChar);
begin
  {Load the library}
  Lib := LoadLibrary(DLLName);
  {If it wasn't loaded succesfully, then call error with em_DLLNotFound}
  if Lib < 32 then
    Status := em_DLLNotFound
  else
    TApplication.Init(AName);
end;

destructor TPBarApp.Done;
begin
  inherited Done;
  {Free the library}
  FreeLibrary(Lib);
end;

procedure TPBarApp.InitMainWindow;
begin
  MainWindow := New(PPBarWindow, Init(nil, MakeIntResource(100)));
end;

procedure TPBarApp.Error(ErrorCode: Integer);
begin
  case ErrorCode of
    em_DLLNotFound:
      begin
        {if the dll could not be loaded display the message and fail the app}
        MessageBox(0, DLLName + ' not found. Please make sure that Windows will find this file!', 'Hard Error',
          mb_Ok or mb_IconStop);
        Halt(ErrorCode);
      end;
  else
    TApplication.Error(ErrorCode);
  end;
end;

var
  App: TPBarApp;

begin
  App.Init('PBar Demo');
  App.Run;
  App.Done;
end.
